@@ -1,8 +1,8 @@
 #! /usr/bin/python -O
-# -*- coding: ISO-8859-15 -*-
+# -*- coding: utf-8 -*-
 #
 # To-do list manager.
-# Copyright (C) 2006-2008 MiKael NAVARRO
+# Copyright (C) 2006-2009 MiKael NAVARRO
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 #
 
 """yaGTD
-Copyright (C) 2006-2008 MiKael NAVARRO
+Copyright (C) 2006-2009 MiKael NAVARRO
 
 A primitive Getting Things Done to-do list manager.
 Cmd version: line oriented command interpreters.
@@ -29,8 +29,8 @@ Cmd version: line oriented command interpreters.
 
 # Specific variables for pydoc
 __author__ = "MiKael Navarro <klnavarro@gmail.com>"
-__date__ = "Fri May 23 2008"
-__version__ = "0.2.4"
+__date__ = "November 2009"
+__version__ = "0.3.0"
 
 # Include directives
 import os
@@ -153,8 +153,10 @@ class GTD(cmd.Cmd):
         self.todo = ToDo()  # the to-do list is here!
 
         self.todotxt = TODO_TXT
+        self.donetxt = DONE_TXT
 
         self.colorize = False
+        self.tasks_selected = []
 
     #
     # Private functions.
@@ -309,13 +311,9 @@ class GTD(cmd.Cmd):
     def _disp(self, task):
         """Display the 'id' and a summary of the 'task'."""
 
-        task_line = task['title']
-        if __debug__:
-            task_line = self._dump_line(task)
+        task_line = self._dump_line(task)
 
-        id_str = "%3d:" % task['id']
-        if __debug__:
-            id_str = "%3d:(%f)" % (task['id'], task.priority())
+        id_str = "%3d:(%f)" % (task['id'], task.priority())
         if self.colorize:  # colorize #id
             id_str = COLOR_CODES['cyan'] + id_str + DEFAULT_COLOR
         
@@ -329,7 +327,14 @@ class GTD(cmd.Cmd):
             if task.has_key(attrib) and task[attrib]:
                 if s: s = s + "\n"
                 s = s + attrib.capitalize() + ": "
-                s = s + str(task[attrib])
+                if attrib == 'urgency':
+                    s = s + str(gtd.URGENCY[task[attrib]])
+                elif attrib == 'importance':
+                    s = s + str(gtd.IMPORTANCE[task[attrib]])
+                elif attrib == 'complete':
+                    s = s + str(task[attrib]) + '%'
+                else:
+                    s = s + str(task[attrib])
         return s
 
     def _add(self, line):
@@ -366,6 +371,9 @@ class GTD(cmd.Cmd):
 
             # And, replace it into the to-do list
             self.todo[i] = task
+            
+            # And, set the start date if none
+            self.do_append("%d S:%s" % (idx, datetime.datetime.now().strftime("%Y-%m-%d")))
 
     def _search(self, regexp, completed=False, quiet=False):
         """Retrieve tasks matching given regexp.
@@ -473,8 +481,8 @@ class GTD(cmd.Cmd):
                 if task['recurrence']:
                     if not task['due']:
                         task['due'] = datetime.datetime.now()
-                    task['start'] = task['due']  # reinit start date
-                    task['due'] = task['start'] + task['recurrence']
+                    task['start'] = task['start'] + task['recurrence']  # reinit start date
+                    task['due'] = task['due'] + task['recurrence']
                     print "Task #%d (%s) rescheduled (%s)." % (idx, task['title'], task['due'])
                 else:
                     task['end'] = datetime.datetime.now()
@@ -581,12 +589,6 @@ class GTD(cmd.Cmd):
         for taskid in tasks:
             self.do_modify('%d %s' % (taskid, action))
 
-    def do_archive(self, line):
-        """Archive completed tasks:
-        GTD> archive"""
-
-        print "Not Yet Implemented!"
-
     #
     # Specific GTD commands.
     #
@@ -621,7 +623,7 @@ class GTD(cmd.Cmd):
         # Parse command line
         idx, reference = self._parse_args(id_ref)
 
-        if idx:
+        if idx and reference:
             self.do_append("%d ref:%s" % (idx, reference))
 
     #
@@ -635,7 +637,7 @@ class GTD(cmd.Cmd):
         # Parse command line
         idx, level = self._parse_args(id_level)
 
-        if idx and re.match(DIGIT_MATCH, level):
+        if idx and level and re.match(DIGIT_MATCH, level):
             self.do_modify("%d U:%d" % (idx, int(level)))
 
     def do_importance(self, id_level):
@@ -645,7 +647,7 @@ class GTD(cmd.Cmd):
         # Parse command line
         idx, level = self._parse_args(id_level)
 
-        if idx and re.match(DIGIT_MATCH, level):
+        if idx and level and re.match(DIGIT_MATCH, level):
             self.do_modify("%d I:%d" % (idx, int(level)))
 
     def do_complete(self, id_percent):
@@ -655,7 +657,7 @@ class GTD(cmd.Cmd):
         # Parse command line
         idx, percent = self._parse_args(id_percent)
 
-        if idx and re.match(NUMBER_MATCH, percent):
+        if idx and percent and re.match(NUMBER_MATCH, percent):
             self.do_modify("%d C:%d" % (idx, int(percent)))
 
     def do_time(self, id_time):
@@ -665,7 +667,7 @@ class GTD(cmd.Cmd):
         # Parse command line
         idx, time = self._parse_args(id_time)
 
-        if idx and re.match(TIMEDELTA_MATCH, time):
+        if idx and time and re.match(TIMEDELTA_MATCH, time):
             self.do_modify("%d T:%s" % (idx, time))
 
     def do_recurrence(self, id_rec):
@@ -675,37 +677,37 @@ class GTD(cmd.Cmd):
         # Parse command line
         idx, recurrence = self._parse_args(id_rec)
 
-        if idx and re.match(TIMEDELTA_MATCH, recurrence):
+        if idx and recurrence and re.match(TIMEDELTA_MATCH, recurrence):
             self.do_modify("%d R:%s" % (idx, recurrence))
 
     def do_start(self, id_date):
         """Set the Start / creation date:
-        GTD> start YYYY-MM-DD"""
+        GTD> start #id YYYY-MM-DD"""
 
         # Parse command line
         idx, date = self._parse_args(id_date)
 
-        if idx and re.match(DATE_MATCH, date):
+        if idx and date and re.match(DATE_MATCH, date):
             self.do_modify("%d S:%s" % (idx, date))
 
     def do_due(self, id_date):
         """Set the Due (target) date:
-        GTD> due YYYY-MM-DD"""
+        GTD> due #id YYYY-MM-DD"""
 
         # Parse command line
         idx, date = self._parse_args(id_date)
 
-        if idx and re.match(DATE_MATCH, date):
+        if idx and date and re.match(DATE_MATCH, date):
             self.do_modify("%d D:%s" % (idx, date))
 
     def do_end(self, id_date):
         """Set the End, closure date:
-        GTD> end YYYY-MM-DD"""
+        GTD> end #id YYYY-MM-DD"""
 
         # Parse command line
         idx, date = self._parse_args(id_date)
 
-        if idx and re.match(DATE_MATCH, date):
+        if idx and date and re.match(DATE_MATCH, date):
             self.do_modify("%d E:%s" % (idx, date))
 
     #
@@ -786,23 +788,69 @@ class GTD(cmd.Cmd):
                 print self._show(task)
 
     def do_sort(self, nb):
-        """Sort #nb tasks by priority:
+        """Sort #nb tasks by priority (appear only after start date):
         GTD> sort [#nb]"""
 
         # Parse command line
         nb = self._parse_args(nb)[0]
 
         tasks = [ t for t in self.todo.sort() if t['complete'] < 100 ]
+
+        # Appear only after start date
+        today = datetime.datetime.now()
+        tasks = [ t for t in tasks if t['start'] <= today ]
+
         if nb:  # display only nb tasks
             tasks = tasks[:nb]
 
         for t in tasks:
             print self._disp(t)
 
-    do_listpri = do_sort
+    def do_today(self, nb):
+        """Sort #nb tasks by priority, and shows only today
+        (the time now before and after start date):
+        GTD> today [#nb]"""
+
+        # Parse command line
+        nb = self._parse_args(nb)[0]
+
+        tasks = [ t for t in self.todo.sort() if t['complete'] < 100 ]
+
+        # Appear only after start date
+        today = datetime.datetime.now()
+        tasks = [ t for t in tasks if t['start'] <= today ]
+
+        if nb: # display only nb tasks
+            tasks = tasks[:nb]
+
+        for t in tasks:
+            if t['due']:
+                if t['due'] <= today:
+                    print self._disp(t)
+            else:
+                print self._disp(t) 
+            
+    def do_listpri(self, regexp):
+        """Sort tasks matching given regexp (appear only after start date):
+        GTD> listpri [regexp]"""
+
+        todos = [ t for t in self.todo.sort() if t['complete'] < 100 ] 
+
+        # Appear only after start date
+        today = datetime.datetime.now()
+        todos = [ t for t in todos if t['start'] <= today ]
+
+        # Then, retrieve the pattern (into full desc tasks)
+        expr = re.compile(regexp, re.IGNORECASE)
+        tasks = [ t for t in todos if expr.search(self._dump_line(t)) ]
+
+        for t in tasks:
+            print self._disp(t)
+
     
     def do_order(self, nb_attr):
-        """Order #nb tasks by context/project/status/reference (and priority):
+        """Order #nb tasks by context/project/status/reference and priority
+        (appear only after start date):
         GTD> order #nb [context|project|status|reference]"""
 
         # Parse command line
@@ -819,6 +867,10 @@ class GTD(cmd.Cmd):
 
             if not tasks: continue  # empty!
 
+            # Appear only after start date
+            today = datetime.datetime.now()
+            tasks = [ t for t in tasks if t['start'] <= today ]
+
             # Section title
             print self._colorize(eval(attr.upper() + '_CHAR') + c.capitalize())
 
@@ -850,7 +902,45 @@ class GTD(cmd.Cmd):
             print "%d%%%s" % (percent, self._colorize(next))
                             
     do_summary = do_status
-            
+
+    #
+    # Select/monitor.
+    #
+
+    def do_select(self, cmd):
+        """ Select a task:
+        GTD> select #id"""
+
+        # Parse command line
+        idx = self._parse_args(cmd)[0]
+
+        task = self.todo.find('id', idx)
+        if not task:
+            print "Task not found"
+            return
+
+        self.tasks_selected.append(idx)
+
+    def do_deselect(self, cmd):
+        """ Deselect a task:
+        GTD> deselect #id"""
+
+        # Parse command line
+        idx = self._parse_args(cmd)[0]
+
+        if idx not in self.tasks_selected:
+            print "Task not found"
+            return
+
+        self.tasks_selected.remove(idx)
+
+    def do_selected(self, line=None):
+        """Show selected tasks sorted by priority:
+        GTD> selected"""
+
+        for t in (self.todo.find("id", idx) for idx in self.tasks_selected):
+            print self._disp(t)
+
     #
     # Re-search.
     #
@@ -910,6 +1000,28 @@ class GTD(cmd.Cmd):
         except IOError, err:
             print err  # and continue
 
+    def do_archive(self, donetxt):
+        """Archive completed tasks:
+        GTD> archive [path/to/done.txt]"""
+
+        if donetxt == "": donetxt = DONE_TXT
+
+        # List done/close tasks
+        done_lst = [ t for t in self.todo if t['complete'] >= 100 ]
+
+        try:
+            f = open(donetxt, 'w')
+            try:
+                for t in done_lst:
+                    f.write(self._dump_line(t) + "\n")
+                # If all okey, remove done/close tasks from current list
+                self.todo = [ t for t in self.todo if t['complete'] < 100 ]
+            finally:
+                f.close()
+                print "%d tasks archived as '%s'" % (len(done_lst), donetxt)
+        except IOError, err:
+            print err  # and continue
+
     def do_print(self, rest):
         """Export into printable format (ReST):
         GTD> print [path/to/todo.rest]"""
@@ -968,7 +1080,35 @@ class GTD(cmd.Cmd):
 
         except IOError, err:
             print err  # and continue
-            
+   
+    #
+    # Help/usage.
+    #
+
+    def do_usage(self, line=None):
+        """Show language reference:
+        GTD> usage"""
+
+        print """Important commands:
+    sort, order, list, listall, status
+    edit, save
+Options:
+    U: urgency (5=today,4=this_week,3=this_month,2=this_quarter,1=this_year)
+    I: importance (5=crucial,4=high,3=normal,2=low,1=someday)
+    C: percent complete
+    T: time required T:n[WDHM]
+    R: recurrence R:n[WDH]
+    S: start/creation date
+    D: due (target) date
+    E: end, closure date
+    @context
+    p:Project
+    !status
+    ref:reference
+Type 'help' or '?' for more commands/options."""
+
+    do_langref = do_usage
+         
     #
     # Quit.
     #
